//
//  ViewController.swift
//  Still
//
//  Created by Mark Meretzky on 12/12/14.
//  Copyright (c) 2014 Mark Meretzky. All rights reserved.
//

import UIKit;
import MobileCoreServices;	//for kUTTypeImage
import AssetsLibrary;		//for writeImageToSavedPhotosAlbum(_:metadata:completionBlock:)

class ViewController: UIViewController, UINavigationControllerDelegate, UIImagePickerControllerDelegate {
	let imagePickerController: UIImagePickerController = UIImagePickerController();

	required init(coder aDecoder: NSCoder) {
		super.init(coder: aDecoder);
	}

	//Implant a reference to my own self (i.e., the ViewController)
	//into the newborn View.

	override func loadView() {
		view = View(viewController: self);
	}

	override func viewDidLoad() {
		super.viewDidLoad();
		// Do any additional setup after loading the view, typically from a nib.
	}

	//Called when the user presses the "Pick an image" button in the big white View.

	func touchUpInside(button: UIButton!) {
		//Is there a source from which we can pick an image?
		
		if !UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.Camera)
			&& !UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.PhotoLibrary)
			&& !UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.SavedPhotosAlbum) {

			let noSourceController: UIAlertController = UIAlertController(
				title: "No Source",
				message: "No source is currently available from which to pick an image.",
				preferredStyle: UIAlertControllerStyle.Alert);

			let alertAction: UIAlertAction = UIAlertAction(
				title: "OK",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					println("OK button pressed");
				});

			noSourceController.addAction(alertAction);
			presentViewController(noSourceController, animated: true, completion: {
				println("viewDidAppear method of noSourceController has been called.");
			});
			return;
		}

		let sourceController: UIAlertController = UIAlertController(
			title: "Select a source",
			message: "Where should the image come from?",
			preferredStyle: UIAlertControllerStyle.ActionSheet);

		if UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.Camera) {
			let cameraAction: UIAlertAction = UIAlertAction(
				title: "Camera",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					self.imagePickerController.sourceType = UIImagePickerControllerSourceType.Camera;
					self.typeImage();
				});
			sourceController.addAction(cameraAction);
		}

		if UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.PhotoLibrary) {
			let photoLibraryAction: UIAlertAction = UIAlertAction(
				title: "Photo Library",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					self.imagePickerController.sourceType = UIImagePickerControllerSourceType.PhotoLibrary;
					self.typeImage();
				});
			sourceController.addAction(photoLibraryAction);
		}
		
		if UIImagePickerController.isSourceTypeAvailable(UIImagePickerControllerSourceType.SavedPhotosAlbum) {
			let savedPhotosAlbumAction: UIAlertAction = UIAlertAction(
				title: "Saved Photos Album",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					self.imagePickerController.sourceType = UIImagePickerControllerSourceType.SavedPhotosAlbum;
					self.typeImage();
				});
			sourceController.addAction(savedPhotosAlbumAction);
		}
		
		let cancelAction: UIAlertAction = UIAlertAction(
			title: "Cancel",
			style: UIAlertActionStyle.Cancel,
			handler: {(alertAction: UIAlertAction!) -> Void in
			});
		sourceController.addAction(cancelAction);

		presentViewController(sourceController, animated: true, completion: nil);
	}
	
	func typeImage() {
		if imagePickerController.sourceType == UIImagePickerControllerSourceType.Camera {
			//Can the camera take a still photo (kUTTypeImage vs. kUTTypeMovie)?
			//Must convert array of AnyObject into array of String before it can be searched with find.
			let mediaTypes: [AnyObject]? =
			UIImagePickerController.availableMediaTypesForSourceType(imagePickerController.sourceType);
			if mediaTypes == nil || find(mediaTypes! as [String], kUTTypeImage) == nil {
				println("\(kUTTypeImage) is not available");
				return;
			}
			imagePickerController.mediaTypes = [kUTTypeImage]; //Display user interface for taking still photos.
			imagePickerController.cameraCaptureMode = UIImagePickerControllerCameraCaptureMode.Photo; //vs. .Video
			
			/*
			//Is the rear camera working?
			if !UIImagePickerController.isCameraDeviceAvailable(UIImagePickerControllerCameraDevice.Rear) { //vs. .Front
			println("rear camera is not available");
			return;
			}
			imagePickerController.cameraDevice = UIImagePickerControllerCameraDevice.Rear;
			*/
			let frontRearController: UIAlertController = UIAlertController(title: "Choose a Camera",
				message: nil,
				preferredStyle: UIAlertControllerStyle.ActionSheet);
			
			let frontAction: UIAlertAction = UIAlertAction(title: "Front",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					println("Front button pressed");
					self.imagePickerController.cameraDevice = UIImagePickerControllerCameraDevice.Front;
			});
			frontRearController.addAction(frontAction);
			
			let rearAction: UIAlertAction = UIAlertAction(title: "Rear",
				style: UIAlertActionStyle.Default,
				handler: {(alertAction: UIAlertAction!) -> Void in
					println("Rear button pressed");
					self.imagePickerController.cameraDevice = UIImagePickerControllerCameraDevice.Rear;
			});
			frontRearController.addAction(rearAction);
			presentViewController(frontRearController, animated: true, completion: nil);
			
			//Does the rear camera have a flash?
			
			if UIImagePickerController.isFlashAvailableForCameraDevice(UIImagePickerControllerCameraDevice.Rear) {
				imagePickerController.cameraFlashMode = UIImagePickerControllerCameraFlashMode.Auto;
			}
			
			imagePickerController.showsCameraControls = true;
			imagePickerController.allowsEditing = false; //Editing would result in photo of only 640 x 640 pixels.
		}
		imagePickerController.delegate = self;
		
		imagePickerController.modalPresentationStyle = UIModalPresentationStyle.FullScreen;
		imagePickerController.modalTransitionStyle = UIModalTransitionStyle.CoverVertical;
		presentViewController(imagePickerController, animated: true, completion: nil);
	}

	//This method of the UIImagePickerControllerDelegate is called
	//when the user presses the camera's "Use Photo" button.

	func imagePickerController(picker: UIImagePickerController,
		didFinishPickingMediaWithInfo info: [NSObject: AnyObject]) {

		//If we captured a still image,
		let mediaType: CFString? = info[UIImagePickerControllerMediaType] as CFString?;
		if mediaType != nil && mediaType! == kUTTypeImage {

			//Print the image's width and height.
			let metadata: [NSObject: AnyObject]? =
				info[UIImagePickerControllerMediaMetadata] as [NSObject: AnyObject]?;
			if metadata != nil {
				//Exchangeable image file format
				let exif: [NSObject: AnyObject]? =
					metadata!["{Exif}"] as [NSObject: AnyObject]?;
				if exif != nil {
					let width: Int? = exif!["PixelXDimension"] as Int?;
					let height: Int? = exif!["PixelYDimension"] as Int?;
					if width != nil && height != nil {
						println("dimensions in pixels = \(width!) × \(height!)");
					}
				}
			}

			//Is there an image to save?
			var imageToSave: UIImage?;
			let editedImage: UIImage? = info[UIImagePickerControllerEditedImage] as UIImage?;
			if editedImage != nil {
				imageToSave = editedImage!;
			} else {
				let originalImage: UIImage? =
					info[UIImagePickerControllerOriginalImage] as UIImage?;
				if originalImage != nil {
					imageToSave = originalImage!
				}
			}

			//Save the image to the Camera Roll album.
			if imageToSave == nil {
				println("could not find original image or edited image");
			} else {
				let assetsLibrary: ALAssetsLibrary = ALAssetsLibrary();
				assetsLibrary.writeImageToSavedPhotosAlbum(
					imageToSave!.CGImage,
					metadata: metadata,
					completionBlock: {(url: NSURL!, error: NSError!) in
						println("image written: URL = \(url), error = \(error)");
					}
				);
			}
		}

		dismissViewControllerAnimated(true, completion: nil);
	}

	//This method of the UIImagePickerControllerDelegate is called
	//when the user presses the camera's "Cancel" button.

	func imagePickerControllerDidCancel(picker: UIImagePickerController) {
		println("didCancel");
		dismissViewControllerAnimated(true, completion: nil);
	}

	override func didReceiveMemoryWarning() {
		super.didReceiveMemoryWarning()
		// Dispose of any resources that can be recreated.
	}

}

